@@ -53,6 +53,11 @@ impl<M: ManagedTypeApi> ManagedTypeHelper<M> {
         ManagedVec::new(self.api.clone())
     }
 
+    #[inline]
+    pub fn managed_vec_from_single_item<T: ManagedVecItem<M>>(&self, item: T) -> ManagedVec<M, T> {
+        ManagedVec::from_single_item(self.api.clone(), item)
+    }
+
     #[inline]
     pub fn managed_vec_from<T: ManagedVecItem<M>, V: ManagedInto<M, ManagedVec<M, T>>>(
         &self,
