@@ -1,12 +1,14 @@
-use crate::api::{EndpointArgumentApi, ErrorApi};
-use crate::*;
+use crate::{
+    api::{EndpointArgumentApi, ErrorApi, ManagedTypeApi},
+    *,
+};
 use elrond_codec::*;
 
-#[inline]
+#[inline(always)]
 pub fn load_single_arg<AA, T>(api: AA, index: i32, arg_id: ArgId) -> T
 where
     T: TopDecode,
-    AA: EndpointArgumentApi + ErrorApi + Clone + 'static,
+    AA: ManagedTypeApi + EndpointArgumentApi + ErrorApi,
 {
     T::top_decode_or_exit(
         ArgDecodeInput::new(api.clone(), index),
@@ -18,18 +20,17 @@ where
 #[inline(always)]
 fn load_single_arg_exit<AA>(ctx: (AA, ArgId), de_err: DecodeError) -> !
 where
-    AA: EndpointArgumentApi + ErrorApi + 'static,
+    AA: ManagedTypeApi + EndpointArgumentApi + ErrorApi,
 {
     let (api, arg_id) = ctx;
-    signal_arg_de_error(&api, arg_id, de_err)
+    signal_arg_de_error(api, arg_id, de_err)
 }
 
 /// It's easier to generate code from macros using this function, instead of the DynArg method.
 #[inline]
-pub fn load_dyn_arg<I, D, T>(loader: &mut D, arg_id: ArgId) -> T
+pub fn load_dyn_arg<I, T>(loader: &mut I, arg_id: ArgId) -> T
 where
-    I: TopDecodeInput,
-    D: DynArgInput<I>,
+    I: DynArgInput,
     T: DynArg,
 {
     T::dyn_load(loader, arg_id)
