@@ -4,9 +4,8 @@ use crate::{
     api::{EndpointFinishApi, ErrorApi, ManagedTypeApi, StorageReadApi, StorageWriteApi},
     io::EndpointResult,
     storage::{storage_get, storage_get_len, storage_set, StorageKey},
-    types::{Address, MultiResultVec},
+    types::{ManagedAddress, ManagedVec, MultiResultVec},
 };
-use alloc::vec::Vec;
 
 const ADDRESS_TO_ID_SUFFIX: &[u8] = b"_address_to_id";
 const ID_TO_ADDRESS_SUFFIX: &[u8] = b"_id_to_address";
@@ -42,7 +41,7 @@ impl<SA> UserMapper<SA>
 where
     SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
 {
-    fn get_user_id_key(&self, address: &Address) -> StorageKey<SA> {
+    fn get_user_id_key(&self, address: &ManagedAddress<SA>) -> StorageKey<SA> {
         let mut user_id_key = self.base_key.clone();
         user_id_key.append_bytes(ADDRESS_TO_ID_SUFFIX);
         user_id_key.append_item(address);
@@ -64,16 +63,16 @@ where
 
     /// Yields the user id for a given address.
     /// Will return 0 if the address is not known to the contract.
-    pub fn get_user_id(&self, address: &Address) -> usize {
+    pub fn get_user_id(&self, address: &ManagedAddress<SA>) -> usize {
         storage_get(self.api.clone(), &self.get_user_id_key(address))
     }
 
-    fn set_user_id(&self, address: &Address, id: usize) {
+    fn set_user_id(&self, address: &ManagedAddress<SA>, id: usize) {
         storage_set(self.api.clone(), &self.get_user_id_key(address), &id);
     }
 
     /// Yields the user address for a given id, if the id is valid.
-    pub fn get_user_address(&self, id: usize) -> Option<Address> {
+    pub fn get_user_address(&self, id: usize) -> Option<ManagedAddress<SA>> {
         let key = self.get_user_address_key(id);
         // TODO: optimize, storage_load_managed_buffer_len is currently called twice
 
@@ -86,23 +85,23 @@ where
 
     /// Yields the user address for a given id.
     /// Will cause a deserialization error if the id is invalid.
-    pub fn get_user_address_unchecked(&self, id: usize) -> Address {
+    pub fn get_user_address_unchecked(&self, id: usize) -> ManagedAddress<SA> {
         storage_get(self.api.clone(), &self.get_user_address_key(id))
     }
 
     /// Yields the user address for a given id, if the id is valid.
     /// Otherwise returns the zero address (0x000...)
-    pub fn get_user_address_or_zero(&self, id: usize) -> Address {
+    pub fn get_user_address_or_zero(&self, id: usize) -> ManagedAddress<SA> {
         let key = self.get_user_address_key(id);
         // TODO: optimize, storage_load_managed_buffer_len is currently called twice
         if storage_get_len(self.api.clone(), &key) > 0 {
             storage_get(self.api.clone(), &key)
         } else {
-            Address::zero()
+            ManagedAddress::zero(self.api.clone())
         }
     }
 
-    fn set_user_address(&self, id: usize, address: &Address) {
+    fn set_user_address(&self, id: usize, address: &ManagedAddress<SA>) {
         storage_set(self.api.clone(), &self.get_user_address_key(id), address);
     }
 
@@ -117,7 +116,7 @@ where
 
     /// Yields the user id for a given address, or creates a new user id if there isn't one.
     /// Will safely keep the user count in sync.
-    pub fn get_or_create_user(&self, address: &Address) -> usize {
+    pub fn get_or_create_user(&self, address: &ManagedAddress<SA>) -> usize {
         let mut user_id = self.get_user_id(address);
         if user_id == 0 {
             let mut user_count = self.get_user_count();
@@ -132,32 +131,35 @@ where
 
     /// Tries to insert a number of addresses.
     /// Calls a lambda function for each, with the new user id and whether of nor the user was already present.
-    pub fn get_or_create_users<F: FnMut(usize, bool)>(
+    pub fn get_or_create_users<AddressIter, F>(
         &self,
-        addresses: &[Address],
+        address_iter: AddressIter,
         mut user_id_lambda: F,
-    ) {
+    ) where
+        AddressIter: Iterator<Item = ManagedAddress<SA>>,
+        F: FnMut(usize, bool),
+    {
         let mut user_count = self.get_user_count();
-        for address in addresses {
-            let mut user_id = self.get_user_id(address);
+        for address in address_iter {
+            let mut user_id = self.get_user_id(&address);
             if user_id > 0 {
                 user_id_lambda(user_id, false);
             } else {
                 user_count += 1;
                 user_id = user_count;
-                self.set_user_id(address, user_id);
-                self.set_user_address(user_id, address);
+                self.set_user_id(&address, user_id);
+                self.set_user_address(user_id, &address);
                 user_id_lambda(user_id, true);
             }
         }
         self.set_user_count(user_count);
     }
 
-    /// Loads all addresses from storage and places them in a Vec.
+    /// Loads all addresses from storage and places them in a ManagedVec.
     /// Can easily consume a lot of gas.
-    pub fn get_all_addresses(&self) -> Vec<Address> {
+    pub fn get_all_addresses(&self) -> ManagedVec<SA, ManagedAddress<SA>> {
         let user_count = self.get_user_count();
-        let mut result = Vec::with_capacity(user_count);
+        let mut result = ManagedVec::new(self.api.clone());
         for i in 1..=user_count {
             result.push(self.get_user_address_or_zero(i));
         }
@@ -171,14 +173,16 @@ impl<SA> EndpointResult for UserMapper<SA>
 where
     SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
 {
-    type DecodeAs = MultiResultVec<Address>;
+    type DecodeAs = MultiResultVec<ManagedAddress<SA>>;
 
     fn finish<FA>(&self, api: FA)
     where
         FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
     {
-        let addr_vec = self.get_all_addresses();
-        MultiResultVec::<Address>::from(addr_vec).finish(api);
+        let all_addresses = self.get_all_addresses();
+        for address in all_addresses.into_iter() {
+            address.finish(api.clone());
+        }
     }
 }
 
@@ -188,7 +192,7 @@ where
     SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
 {
     fn type_name() -> TypeName {
-        crate::types::MultiResultVec::<Address>::type_name()
+        crate::types::MultiResultVec::<ManagedAddress<SA>>::type_name()
     }
 
     fn is_multi_arg_or_result() -> bool {
